module Core.Couch where

import Core.Types

import Control.Monad.Reader (MonadIO, lift, ask)
import Data.Aeson
import Network.HTTP.Req     as Req

import qualified Data.Text                  as T
import qualified Data.ByteString.Char8      as B
import qualified Data.ByteString.Lazy.Char8 as LB

getAllDBs :: PurrAction LbsResponse
getAllDBs = do
  host <- dbHost
  user <- dbUser
  pw   <- dbPassword
  req
    GET
    (http host /: "_all_dbs")
    NoReqBody
    lbsResponse $
    basicAuthUnsafe user pw
      <>
    Req.port 5984

createDB :: (FromJSON a) => T.Text -> PurrAction (JsonResponse a)
createDB dbName  = do
  host <- dbHost
  user <- dbUser
  pw   <- dbPassword
  req
    PUT
    (http host /: dbName)
    NoReqBody
    jsonResponse $
    basicAuthUnsafe user pw
      <>
    Req.port 5984

getAllDocs :: PurrAction LbsResponse
getAllDocs = do
  host   <- dbHost
  dbName <- confDb
  user   <- dbUser
  pw     <- dbPassword
  req
    GET
    (http host /: dbName)
    NoReqBody
    lbsResponse $
    basicAuthUnsafe user pw
      <>
    Req.port 5984

confDb :: PurrAction T.Text
confDb = do
  conf <- lift ask
  return $ T.pack $ dataDB conf

dbHost :: PurrAction T.Text
dbHost = do
  conf <- lift ask
  return $ T.pack $ couchHost conf

dbPort :: PurrAction Int
dbPort = do
  conf <- lift ask
  return $ couchPort conf

dbPassword :: PurrAction B.ByteString
dbPassword = do
  conf <- lift ask
  return $ B.pack $ couchPassword conf

dbUser :: PurrAction B.ByteString
dbUser = do
  conf <- lift ask
  return $ B.pack $ couchUsername conf
